from arcadia_pycolor.hexcode import HexCode

# Primary colors.
aegean = HexCode("aegean", "#5088C5")
amber = HexCode("amber", "#F28360")
seaweed = HexCode("seaweed", "#3B9886")
canary = HexCode("canary", "#F7B846")
aster = HexCode("aster", "#7A77AB")
rose = HexCode("rose", "#F898AE")
vital = HexCode("vital", "#73B5E3")
tangerine = HexCode("tangerine", "#FFB984")
lime = HexCode("lime", "#97CD78")
dragon = HexCode("dragon", "#C85152")
oat = HexCode("oat", "#F5E4BE")
wish = HexCode("wish", "#BABEE0")

# Secondary colors.
sky = HexCode("sky", "#C6E7F4")
dress = HexCode("dress", "#F8C5C1")
taupe = HexCode("taupe", "#DBD1C3")
denim = HexCode("denim", "#B6C8D4")
sage = HexCode("sage", "#B5BEA4")
marine = HexCode("marine", "#8A99AD")
mars = HexCode("mars", "#DA9085")
shell = HexCode("shell", "#EDE0D6")

# Neutral colors.
gray = HexCode("gray", "#EBEDE8")
chateau = HexCode("chateau", "#BAB0A8")
bark = HexCode("bark", "#8F8885")
slate = HexCode("slate", "#43413F")
charcoal = HexCode("charcoal", "#484B50")
crow = HexCode("crow", "#292928")
forest = HexCode("forest", "#596F74")
pitch = HexCode("pitch", "#09090A")

# Background colors.
parchment = HexCode("parchment", "#FDF8F2")
zephyr = HexCode("zephyr", "#F4FBFF")
lichen = HexCode("lichen", "#F7FBEF")
dawn = HexCode("dawn", "#F8F4F1")

# Shades.
lapis = HexCode("lapis", "#2B65A1")
dusk = HexCode("dusk", "#094468")
cinnabar = HexCode("cinnabar", "#9E3F41")
melon = HexCode("melon", "#FFD0B0")
sun = HexCode("sun", "#FFD364")
mustard = HexCode("mustard", "#D68D22")
umber = HexCode("umber", "#A85E28")
iris = HexCode("iris", "#DCDFEF")
tanzanite = HexCode("tanzanite", "#54448C")
glass = HexCode("glass", "#C3E2DB")
teal = HexCode("teal", "#6FBCAD")
asparagus = HexCode("asparagus", "#2A6B5E")
depths = HexCode("depths", "#09473E")
putty = HexCode("putty", "#FFE3D4")
candy = HexCode("candy", "#E2718F")
azalea = HexCode("azalea", "#C14C70")
stone = HexCode("stone", "#EDE6DA")
mud = HexCode("mud", "#635C5A")
ice = HexCode("ice", "#E6EAED")
dove = HexCode("dove", "#CAD4DB")
cloud = HexCode("cloud", "#ABBAC4")
steel = HexCode("steel", "#687787")
fern = HexCode("fern", "#47784A")
edamame = HexCode("edamame", "#C1E1AE")
matcha = HexCode("matcha", "#71AC5A")
yucca = HexCode("yucca", "#1E4812")

# Gradient colors.
concord = HexCode("concord", "#341E60")
heather = HexCode("heather", "#A96789")
tumbleweed = HexCode("tumbleweed", "#E9A482")
wheat = HexCode("wheat", "#F5DFB2")
shire = HexCode("shire", "#4E7F72")
topaz = HexCode("topaz", "#FFCC7B")
space = HexCode("space", "#282A49")
butter = HexCode("butter", "#FFFDBD")
terracotta = HexCode("terracotta", "#964222")
blush = HexCode("blush", "#FFF3F4")
lilac = HexCode("lilac", "#6862AB")
ghost = HexCode("ghost", "#FCF7FF")
redwood = HexCode("redwood", "#52180A")

# Other Arcadia colors.
brightgray = HexCode("brightgray", "#EAEAEA")
paper = HexCode("paper", "#FCFCFC")

soil = HexCode("soil", "#4D2500")
blossom = HexCode("blossom", "#F5CBE4")
mint = HexCode("mint", "#D1EADF")

# Other named colors.
white = HexCode("white", "#FFFFFF")
red = HexCode("red", "#FF0000")
green = HexCode("green", "#00FF00")
blue = HexCode("blue", "#0000FF")
cyan = HexCode("cyan", "#00FFFF")
magenta = HexCode("magenta", "#FF00FF")
yellow = HexCode("yellow", "#FFFF00")
black = HexCode("black", "#000000")
